@@ -1,9 +1,7 @@
 /*
  * Socket used to communicate
- * Choose either PORT or SERVICE...
  */
-#undef SERVICENAME
-#undef PORTNUMBER
+#undef DEFPORT
 
 /*
  * Hostname of console server
@@ -20,6 +18,11 @@
  */
 #undef PASSWDFILE
 
+/*
+ * Logfile path
+ */
+#undef LOGFILEPATH
+
 /*
  * Number of consoles per child process
  */
